@@ -35,7 +35,8 @@
 #define SUBFOLDER_DIR_NAME_LEN 10
 
 
-static CamelStoreClass *parent_class = NULL;
+//static CamelStoreClass *parent_class = NULL;
+static CamelOfflineStoreClass *parent_class = NULL;
 
 #define CS_CLASS(so) ((CamelStoreClass *)((CamelObject *)(so))->klass)
 
@@ -80,7 +81,7 @@ class_init (CamelExchangeStoreClass *camel_exchange_store_class)
 	CamelStoreClass *camel_store_class =
 		CAMEL_STORE_CLASS (camel_exchange_store_class);
 	
-	parent_class = CAMEL_STORE_CLASS (camel_type_get_global_classfuncs (camel_store_get_type ()));
+	parent_class = CAMEL_OFFLINE_STORE_CLASS (camel_type_get_global_classfuncs (camel_offline_store_get_type ()));
 	
 	/* virtual method overload */
 	camel_service_class->construct = construct;
@@ -127,7 +128,7 @@ camel_exchange_store_get_type (void)
 
 	if (!camel_exchange_store_type) {
 		camel_exchange_store_type = camel_type_register (
-			CAMEL_STORE_TYPE,
+			camel_offline_store_get_type (),
 			"CamelExchangeStore",
 			sizeof (CamelExchangeStore),
 			sizeof (CamelExchangeStoreClass),
@@ -331,14 +332,28 @@ exchange_disconnect (CamelService *service, gboolean clean, CamelException *ex)
 
 static CamelFolder *
 exchange_get_folder (CamelStore *store, const char *folder_name,
-		     guint32 camel_flags, CamelException *ex)
+		     guint32 flags, CamelException *ex)
 {
 	CamelExchangeStore *exch = CAMEL_EXCHANGE_STORE (store);
 	CamelFolder *folder;
 	char *folder_dir;
-
+	
 	RETURN_VAL_IF_NOT_CONNECTED (store, ex, NULL);
+	
+	folder_dir = exchange_path_to_physical (exch->storage_path, folder_name);
+	
+	printf ("folder_name : %s\n", folder_name );
 
+	if (((CamelOfflineStore *) store)->state == CAMEL_OFFLINE_STORE_NETWORK_UNAVAIL) {
+		if (!folder_dir || access (folder_dir, F_OK) != 0) {
+			g_free (folder_dir);
+			printf ("no folder_dir or no write access\n");
+			camel_exception_setv (ex, CAMEL_EXCEPTION_STORE_NO_FOLDER,
+					      _("No such folder %s"), folder_name);
+			return NULL;
+		}
+	}
+	
 	g_mutex_lock (exch->folders_lock);
 	folder = g_hash_table_lookup (exch->folders, folder_name);
 	if (folder) {
@@ -347,16 +362,16 @@ exchange_get_folder (CamelStore *store, const char *folder_name,
 		 */
 		g_mutex_unlock (exch->folders_lock);
 		camel_object_ref (CAMEL_OBJECT (folder));
+		g_free (folder_dir);
 		return folder;
 	}
 
 	folder = (CamelFolder *)camel_object_new (CAMEL_EXCHANGE_FOLDER_TYPE);
 	g_hash_table_insert (exch->folders, g_strdup (folder_name), folder);
 	g_mutex_unlock (exch->folders_lock);
-
-	folder_dir = exchange_path_to_physical (exch->storage_path, folder_name);
+	
 	if (!camel_exchange_folder_construct (folder, store, folder_name,
-					      camel_flags, folder_dir,
+					      flags, folder_dir,
 					      exch->stub, ex)) {
 		g_free (folder_dir);
 		camel_object_unref (CAMEL_OBJECT (folder));
@@ -446,8 +461,7 @@ postprocess_tree (CamelFolderInfo *info)
 			
 
 static CamelFolderInfo *
-exchange_get_folder_info (CamelStore *store, const char *top,
-			  guint32 flags, CamelException *ex)
+exchange_get_folder_info (CamelStore *store, const char *top, guint32 flags, CamelException *ex)
 {
 	CamelExchangeStore *exch = CAMEL_EXCHANGE_STORE (store);
 	GPtrArray *folders, *folder_names, *folder_uris;
@@ -455,7 +469,12 @@ exchange_get_folder_info (CamelStore *store, const char *top,
 	GByteArray *folder_flags;
 	CamelFolderInfo *info;
 	int i;
-
+	
+	if (((CamelOfflineStore *) store)->state == CAMEL_OFFLINE_STORE_NETWORK_UNAVAIL) {
+		camel_exception_set (ex, CAMEL_EXCEPTION_SYSTEM, _("Cannot get folder info in offline mode."));
+		return NULL;
+	}
+	
 	RETURN_VAL_IF_NOT_CONNECTED (store, ex, NULL);
 
 	/* If the backend crashed, don't keep returning an error
@@ -512,6 +531,11 @@ exchange_create_folder (CamelStore *store, const char *parent_name,
 	char *folder_uri;
 	guint32 unread_count, flags;
 	CamelFolderInfo *info;
+	
+	if (((CamelOfflineStore *) store)->state == CAMEL_OFFLINE_STORE_NETWORK_UNAVAIL) {
+		camel_exception_set (ex, CAMEL_EXCEPTION_SYSTEM, _("Cannot create folder in offline mode."));
+		return NULL;
+	}
 
 	if (!camel_stub_send (exch->stub, ex, CAMEL_STUB_CMD_CREATE_FOLDER,
 			      CAMEL_STUB_ARG_FOLDER, parent_name,
@@ -535,6 +559,11 @@ exchange_delete_folder (CamelStore *store, const char *folder_name,
 {
 	CamelExchangeStore *exch = CAMEL_EXCHANGE_STORE (store);
 
+	if (((CamelOfflineStore *) store)->state == CAMEL_OFFLINE_STORE_NETWORK_UNAVAIL) {
+		camel_exception_set (ex, CAMEL_EXCEPTION_SYSTEM, _("Cannot delete folder in offline mode."));
+		return;
+	}
+
 	camel_stub_send (exch->stub, ex, CAMEL_STUB_CMD_DELETE_FOLDER,
 			 CAMEL_STUB_ARG_FOLDER, folder_name,
 			 CAMEL_STUB_ARG_END);
@@ -546,6 +575,11 @@ exchange_rename_folder (CamelStore *store, const char *old_name,
 {
 	CamelExchangeStore *exch = CAMEL_EXCHANGE_STORE (store);
 
+	if (((CamelOfflineStore *) store)->state == CAMEL_OFFLINE_STORE_NETWORK_UNAVAIL) {
+		camel_exception_set (ex, CAMEL_EXCEPTION_SYSTEM, _("Cannot rename folder in offline mode."));
+		return;
+	}
+
 	camel_stub_send (exch->stub, ex, CAMEL_STUB_CMD_RENAME_FOLDER,
 			 CAMEL_STUB_ARG_FOLDER, old_name,
 			 CAMEL_STUB_ARG_FOLDER, new_name,
