@@ -79,9 +79,10 @@ struct _ExchangeAccountPrivate {
 	gboolean uris_use_email;
 
 	char *identity_name, *identity_email, *source_uri, *password_key;
-	char *username, *password, *windows_domain, *ad_server;
+	char *username, *password, *windows_domain, *ad_server, *offline_sync;
+	char *owa_url;
 	E2kAutoconfigAuthPref auth_pref;
-	int ad_limit;
+	int ad_limit, passwd_exp_warn_period;
 
 	EAccountList *account_list;
 	EAccount *account;
@@ -285,6 +286,12 @@ finalize (GObject *object)
 	if (account->priv->ad_server)
 		g_free (account->priv->ad_server);
 
+	if (account->priv->offline_sync)
+		g_free (account->priv->offline_sync);
+
+	if (account->priv->owa_url)
+		g_free (account->priv->owa_url);
+
 	if (account->priv->connect_lock)
 		g_mutex_free (account->priv->connect_lock);
 
@@ -840,6 +847,7 @@ get_password (ExchangeAccount *account, E2kAutoconfig *ac, const char *errmsg)
 		e_passwords_forget_password ("Exchange", account->priv->password_key);
 
 	password = e_passwords_get_password ("Exchange", account->priv->password_key);
+
 	if (!password && exchange_component_is_interactive (global_exchange_component)) {
 		char *prompt;
 
@@ -855,6 +863,13 @@ get_password (ExchangeAccount *account, E2kAutoconfig *ac, const char *errmsg)
 			account->priv->account->source->save_passwd = remember;
 		}
 		g_free (prompt);
+	} 
+	else if (password && !account->priv->account->source->save_passwd) {
+		/* get_password returns the password cached but user has not 
+		 * selected remember password option, forget this password 
+		 * whis is stored temporarily by e2k_validate_user() 
+		 */
+		e_passwords_forget_password ("Exchange", account->priv->password_key);
 	}
 
 	if (password) {
@@ -976,7 +991,7 @@ find_passwd_exp_period (ExchangeAccount *account, E2kGlobalCatalogEntry *entry)
 		max_pwd_age_days = 
 		( max_pwd_age * ONE_HUNDRED_NANOSECOND ) / SECONDS_IN_DAY;
 
-		if (max_pwd_age_days <= PASSWD_EXPIRY_NOTIFICATION_PERIOD) {
+		if (max_pwd_age_days <= account->priv->passwd_exp_warn_period) {
 			/* This we need for changing password */
 			current_passwd = exchange_account_get_password (account);
 			display_passwd_expiry_message (max_pwd_age_days, 
@@ -1664,7 +1679,8 @@ exchange_account_new (EAccountList *account_list, EAccount *adata)
 {
 	ExchangeAccount *account;
 	char *enc_user, *mailbox;
-	const char *param, *proto, *owa_path, *pf_server;
+	const char *param, *proto, *owa_path, *pf_server, *owa_url; 
+	const char *passwd_exp_warn_period, *offline_sync;
 	E2kUri *uri;
 
 	uri = e2k_uri_new (adata->source->url);
@@ -1722,6 +1738,18 @@ exchange_account_new (EAccountList *account_list, EAccount *adata)
 		if (param)
 			account->priv->ad_limit = atoi (param);
 	}
+	
+	passwd_exp_warn_period = e2k_uri_get_param (uri, "passwd_exp_warn_period");
+	if (!passwd_exp_warn_period || !*passwd_exp_warn_period)
+		account->priv->passwd_exp_warn_period = PASSWD_EXPIRY_NOTIFICATION_PERIOD;
+	else
+		account->priv->passwd_exp_warn_period = atoi (passwd_exp_warn_period);
+
+	offline_sync = e2k_uri_get_param (uri, "offline_sync");
+	if (!offline_sync || !*offline_sync)
+		account->priv->offline_sync = g_strdup ("0");
+	else
+		account->priv->offline_sync = g_strdup (offline_sync);
 
 	owa_path = e2k_uri_get_param (uri, "owa_path");
 	if (!owa_path || !*owa_path)
@@ -1734,6 +1762,11 @@ exchange_account_new (EAccountList *account_list, EAccount *adata)
 		pf_server = uri->host;
 
 	proto = e2k_uri_get_param (uri, "use_ssl") ? "https" : "http";
+
+	owa_url = e2k_uri_get_param (uri, "owa_url");
+	if (owa_url)
+		account->priv->owa_url = g_strdup (owa_url); 
+
 	if (uri->port != 0) {
 		account->priv->http_uri_schema =
 			g_strdup_printf ("%s://%%s:%d/%s/%%s/",
